// class ActivityFeed
function ActivityFeed(feedId, feedUrl, maxResults)
{
    this.feedId = feedId;
    this.feedUrl = feedUrl;

    this.maxResults = parseInt(maxResults);
    var thisFeed = this;
    thisFeed.gadgetPrefs = new gadgets.Prefs();

    this.COMMENT_ICON_HTML = '<a class="activity-item-comment-icon" style="visibility:hidden" href="#" title="' +
                             thisFeed.gadgetPrefs.getMsg('gadget.activity.stream.add.comment') + '">&nbsp;</a>';
}

ActivityFeed.prototype.setFeedUrl = function(feedUrl)
{
    this.feedUrl = feedUrl;
};

ActivityFeed.prototype.addItem = function(feedItem, dateFormat)
{
    //dates are in UTC.  Also we group stuff by hours at a bare minimum, so set minutes to 0.  Still displaying the
    //minutes though, since in some languages the time looks crap if there's no minutes.
    var itemDate = Date.parseExact(feedItem.updated, "yyyy-MM-ddTHH:mm:ssZ").setTimezone("UTC").set({minute:0});
    var dateContainer = this.getOrCreateDateContainer(itemDate.toString(dateFormat.container), itemDate.toString(dateFormat.pretty));
    var items = dateContainer.children("ul");
    var item = AJS.$(document.createElement('li'));
    item.addClass("activity-item");
    if (feedItem.icon)
    {
        item.css("backgroundImage", "url('" + feedItem.icon + "')");
    }
    item.html(feedItem.summary);
    items.append(item);
};

ActivityFeed.prototype.getOrCreateDateContainer = function(dateId, date)
{
    var containerId = "dateContainer-" + this.feedId + "-" + dateId;
    var container = AJS.$("#" + containerId);
    if (container.size() == 0)
    {
        var div = AJS.$("#feedContainer-" + this.feedId);

        container = AJS.$(document.createElement("div"));
        container.attr("id", containerId);
        div.append(container);

        var dateLabel = AJS.$(document.createElement("h3"));
        dateLabel.addClass("timestamp");
        dateLabel.html(date);
        container.append(dateLabel);
        var items = AJS.$(document.createElement('ul'));
        items.addClass("activityItems");
        container.append(items);
    }

    return container;
};

ActivityFeed.prototype.getDateFormat = function(minDate, maxDate)
{
    if (minDate == null || maxDate == null)
        return {container:"yyyyMMddHH", pretty:Date.CultureInfo.formatPatterns.monthDay + " - " + Date.CultureInfo.formatPatterns.shortTime};

    var delta = maxDate.getTime() - minDate.getTime();
    var MILLIS_PER_DAY = 1000 * 60 * 60 * 24;
    
    if (delta <= MILLIS_PER_DAY * 2)
        return {container:"yyyyMMddHH", pretty:Date.CultureInfo.formatPatterns.monthDay + " - " + Date.CultureInfo.formatPatterns.shortTime};
    else if (delta <= MILLIS_PER_DAY * 30)
        return {container:"yyyyMMdd",pretty:Date.CultureInfo.formatPatterns.monthDay};
    else
        return {container:"yyyyMM",pretty:Date.CultureInfo.formatPatterns.yearMonth};
};

/**
 * Adds comment forms to all feeds that have a replyTo set.  This should be called before aggregation,
 * because the aggregation depends on it.
 * @param feed The feed to add comment forms to.
 */
ActivityFeed.prototype.addCommentForms = function(feed)
{
    var thisFeed = this;
    for (var i = 0; i < feed.items.length; i++)
    {
        var item = feed.items[i];
        if (item.replyTo)
        {
            item.summary = this.COMMENT_ICON_HTML + item.summary +
                           '<form method="POST" class="activity-item-comment-form">' +
                           '<fieldset>' +
                           '<input type="hidden" name="replyTo" value="' + item.replyTo + '"/>' +
                           '<textarea name="comment" rows="6" cols="40"></textarea>' +
                           '<button type="submit" name="submit">' + thisFeed.gadgetPrefs.getMsg('gadget.activity.stream.save') + '</button>' + 
                           '</fieldset>' +
                           '</form>';
        }
    }
};

ActivityFeed.prototype.populateFeed = function()
{
    var thisFeed = this;

    var handleSuccess = function(data, textStatus)
    {
        var containerId = 'feedContainer-' + thisFeed.feedId;

        var feed = new JAtom(data);
        if (!feed || !feed.items || feed.items.length == 0)
        {
            thisFeed.setDivText(containerId, thisFeed.gadgetPrefs.getMsg('gadget.activity.stream.no.activty.found'));
            AJS.$("#showMoreLink-" + thisFeed.feedId).hide();
            thisFeed.feedPopulated();
            return;
        }

        thisFeed.setDivText(containerId, ''); // remove the loading graphic
        thisFeed.setError(null);
        thisFeed.setWarning(null);
        thisFeed.addCommentForms(feed);

        var maxDate, minDate;
        for (var i = 0; i < feed.items.length; i++)
        {
            var timestamp = Date.parse(feed.items[i].updated);
            if (i == 0)
            {
                maxDate = timestamp;
                minDate = timestamp;
            }
            else if (timestamp.getTime() > maxDate.getTime())
            {
                maxDate = timestamp;
            }
            else if (timestamp.getTime() < minDate.getTime())
                {
                    minDate = timestamp;
                }
        }

        var dateFormat = thisFeed.getDateFormat(minDate, maxDate);

        for (i = 0; i < feed.items.length; i++)
        {
            thisFeed.addItem(feed.items[i], dateFormat);
        }

        // STRM-124: if we asked for 10 results and got back 5, there's no point displaying the showMore link
        if (feed.items.length < thisFeed.maxResults) {
            AJS.$("#showMoreLink-" + thisFeed.feedId).hide();
        }

        // Fade in comment icon when mouse over
        AJS.$(".activity-item:has(.activity-item-comment-icon)").hover(function ()
        {
            // Would love to just use fadeIn fadeOut here, but we can't, because they use
            // display: None at the end, but we want the icon to still take up space
            AJS.$(this).children(".activity-item-comment-icon").css("visibility", "visible").fadeIn("fast");
        }, function()
        {
            AJS.$(this).children(".activity-item-comment-icon").fadeOut("fast", function()
            {
                AJS.$(this).css("visibility", "hidden").css("display", "inline");
            });
        });

        // Highlight everything when mouse is over it
        AJS.$(".activity-item").hover(function()
        {
            AJS.$(this).addClass("hover");
        }, function()
        {
            AJS.$(this).removeClass("hover");
        });

        // Callbacks for clicking on comment icon
        AJS.$(".activity-item-comment-icon").click(function (e) {
            e.preventDefault();
            AJS.$(this).parent().children("form.activity-item-comment-form").slideToggle(function() {
                thisFeed.feedResized();
            });
        }).toggle(function () {
            AJS.$(this).addClass("hide").attr("title", thisFeed.gadgetPrefs.getMsg('gadget.activity.stream.hide'));
        }, function() {
            AJS.$(this).removeClass("hide").attr("title", thisFeed.gadgetPrefs.getMsg('gadget.activity.stream.add.comment'));
        });

        AJS.$(".activity-item-comment-form")
                .children()
                .focus(function()
        {
            AJS.$(this).parent().children("button").attr("accesskey", "s");
        })
                .blur(function()
        {
            AJS.$(this).parent().children("button").removeAttr("accesskey");
        });

        AJS.$(".activity-item-comment-form").submit(function(event)
        {
            event.preventDefault();
            var form = AJS.$(this);

            var commentBody = AJS.$.trim(form.find("textarea").val());
            if(commentBody.length === 0) {
                alert(thisFeed.gadgetPrefs.getMsg('gadget.activity.stream.error.add.comment'));
                return;
            }
            form.children("button").attr("disabled", "true");
            AJS.$.post("/plugins/servlet/streamscomments", AJS.$(this).serialize(), function()
            {
                form.children("textarea").val("").end();
                form.prev(".activity-item-comment-icon").click();
                form.children("button").attr("disabled", "false");
                thisFeed.populateFeed();
            }, "text");
        });
        thisFeed.feedPopulated();
    };

    var handleFailure = function(request, textStatus, errorThrown)
    {
        thisFeed.setDivText('feedContainer-' + thisFeed.feedId, ''); // remove the loading graphic
        thisFeed.setError('Error communicating with server: ' + request.statusText);
        AJS.$("#showMoreLink-" + thisFeed.feedId).hide();
        thisFeed.feedPopulated();
    };

    var sUrl = this.feedUrl;
    if (sUrl.indexOf("?") < 0)
    {
        sUrl += "?";
    }
    else
    {
        sUrl += "&";
    }
    sUrl += "maxResults=" + this.maxResults;

    this.issueRequest("GET", sUrl, null, handleSuccess, handleFailure);

};

ActivityFeed.prototype.issueRequest = function (method, url, data, successCallback, failureCallback)
{
    AJS.$.ajax({
        type: method,
        url: url,
        dataType: "xml",
        success: successCallback,
        error: failureCallback
    });
};

ActivityFeed.prototype.feedPopulated = function()
{

};

ActivityFeed.prototype.feedResized = function()
{

};

ActivityFeed.prototype.setWarning = function(warning)
{
    this.setMessage('#feedWarning-' + this.feedId, warning);
};

ActivityFeed.prototype.setError = function(error)
{
    this.setMessage('#feedError-' + this.feedId, error);
};

ActivityFeed.prototype.setMessage = function(id, text)
{
    var div = AJS.$(id);
    div.text(text);
    text ? div.show() : div.hide();
};

ActivityFeed.prototype.setDivText = function(divId, text)
{
    var div = document.getElementById(divId);
    if (div)
    {
        // The inner div is so that we can add padding in the CSS without having the error box appear when empty
        div.innerHTML = text ? '<div class="inner">' + text + '</div>' : '';
    }
    else
    {
        alert(text);
    }
};

ActivityFeed.toggleStackTraces = function()
{
    AJS.$(".activityFeed .stream-error .stack-trace").toggle();
};

// end class ActivityFeed

function isNumeric(sText)
{
    var ValidChars = "0123456789";
    var Char;

    for (var i = 0; i < sText.length; i++)
    {
        Char = sText.charAt(i);
        if (ValidChars.indexOf(Char) == -1)
        {
            return false;
        }
    }
    return true;
}

function htmlEncode(value)
{
    return AJS.$("<div/>").text(value).html();
}
