AJS.MacroBrowser = {
    // clones the element specified the selector and removes the id attribute
    // TODO: move this to AJS?
    clone: function(selector) {
        return AJS.$(selector).clone().removeAttr("id");
    }
};

AJS.toInit(function($) {

    AJS.activeColumn = 0;

    // Don't initialize if User can't modify Dashboard
    if (!AG.param.get("writable")) {
        return;
    }

    var dashboardDirectoryResourceUrl = AG.param.get("dashboardDirectoryResourceUrl");
    var dashboardResourceUrl = AG.param.get("dashboardResourceUrl");
    var errorStatus = AG.param.get("defaultErrorMessage");
    var browserContentLoading = false;
    var browserContentLoaded = false;

    var addGadgetToDashboard = function(gadgetUrl, callbacks) {
    	var reqParams = {
            type: "POST",
            url: dashboardResourceUrl + ".json",
            contentType: "application/json",
            dataType: "json",
            data: JSON.stringify({
                url: gadgetUrl,
                columnIndex: AJS.activeColumn
            }),
            processData: false
        };
        $.extend(reqParams, callbacks);
        $.ajax(reqParams);
    };

    var insertGadgetIntoLiveDashboard = function(data) {
        AG.DashboardManager.addGadget(data, AJS.activeColumn);
    };

    var callbacks = function(extraData, success, error) {
        return {
            success: function(data, textStatus) {
                success(extraData, data, textStatus);
            },
            error: function(request, textStatus, errorThrown) {
                error(extraData, request, textStatus, errorThrown);
            }
        };
    };

    var addingEntry = function(button) {
        var button = $(button);

        // re-enable all gadget's  buttons
        $('.macro-button-add').removeAttr("disabled");
        $('.macro-button-remove').removeAttr("disabled");
        button.attr("value", AG.param.get("addItNow"));
        button.parents(".macro-list-item").removeAttr("style");
        button.blur();
    };

    var addGadgetToDashboardSuccess = function(button, data, textStatus) {
        insertGadgetIntoLiveDashboard(data);
        var button = $(button);
        button.parents(".macro-list-item").css("background-color", "#fffe83").animate({
            backgroundColor: "transparent"
        },
        2000, "linear",
        function() {
            addingEntry(button);
        });
    };

    var addGadgetToDashboardError = function(button, request, textStatus, errorThrown) {
    	console.log("addItNowError = " + errorThrown);
    	$(button).removeClass("macro-button-add").addClass("macro-button-add-broken");
    	$(button).siblings('.macro-button-remove').removeClass("macro-button-remove").addClass("macro-button-remove-broken");
    	$(button).parents(".macro-list-item").css("background-color", "#faa");

    	// re-enable other gadget's  buttons
    	$('.macro-button-add').removeAttr("disabled");
    	$('.macro-button-remove').removeAttr("disabled");
    	button.blur();
    };

    var removeGadgetFromDirectorySuccess = function(button, data, textStatus) {
        showError(AG.param.get("gadgetRemovedSuccess"));
        var button = $(button);
        button.parents(".macro-list-item").remove();
    };

    var removeGadgetFromDirectoryError = function(button, request, textStatus, errorThrown) {
        if (request.status == 403 || request.status == 401) {
            showError(AG.param.get("dashboardErrorDirectoryPermissions"));
        } else {
            showError(AG.param.get("dashboardErrorFailedToAdd"));
        }
        console.log("addItNowError = " + errorThrown);
        $(button).siblings('.macro-button-add').removeClass("macro-button-add").addClass("macro-button-add-broken");
        $(button).removeClass("macro-button-remove").addClass("macro-button-remove-broken");
        $(button).parents(".macro-list-item").css("background-color", "#faa");
        button.attr("value", AG.param.get("removeFromDirectory"));
        button.blur();
    };

    // Add a new Gadget to Directory by URL
    var addGadgetToDirectory = function() {
        if ($('#add-gadget-url').attr('value') != "") {
            $('#add-gadget-submit').attr("disabled", "disabled").attr("value", AG.param.get("adding"));
            var xhr = $.ajax({
                type: "POST",
                url: dashboardDirectoryResourceUrl,
                data: JSON.stringify({
                    url: $('#add-gadget-url').attr('value')
                }),
                contentType: "application/json",
                processData: false,
                success: function() {
                    var gadgetUrl = $('#add-gadget-url').attr('value');

                    clearBrowser();
                    loadBrowser(false, gadgetUrl);

                },
                error: function(request, textStatus, errorThrown) {
                    if (request.status == 403 || request.status == 401) {
                        showError(AG.param.get("dashboardErrorDirectoryPermissions"));
                    } else {
                        showError(AG.param.get("dashboardErrorFailedToAdd"));
                    }
                    $('#add-gadget-submit').attr("value", AG.param.get("addGadget")).removeAttr("disabled");
                }
            });

            $(xhr).throbber({target: $("#dir-throbber")});
        }
    };

    // Remove a Gadget from the Directory that was added by URL
    var removeGadgetFromDirectory = function(gadgetUri, callbacks) {
        var reqParams = {
            type: "DELETE",
            url: gadgetUri
        };
        $.extend(reqParams, callbacks);
        $.ajax(reqParams);
    };

    // Scroll to a particular gadget
    var scrollToGadget = function(directoryGadgetElementId) {

        var mb = AJS.MacroBrowser.dialog;
        mb.prevPage();

        var gadgetElement = $('#' + directoryGadgetElementId);
        gadgetElement.css("background-color", "#fffe83").animate({ backgroundColor: "transparent" }, 5000, "linear");

        var divOffset = $('.panel-body').offset().top;
        var pOffset = gadgetElement.offset().top;
        var pScroll = pOffset - divOffset;
        $('.panel-body').animate({scrollTop: '+=' + pScroll + 'px'}, 1000);
    };



    // Fill macro Summary Template
    var fillMacroTemplate = function(gadgetDiv, gadget) {

        var onAdd = function(event) {
            if (AG.DashboardManager.getLayout().getGadgets().length >= AG.param.get("maxGadgets")) {
                showError(AG.param.get("dashboardErrorTooManyGadgets"));
                return false;
            }

            var button = $(".macro-button-add", gadgetDiv);
            button.attr("disabled", "disabled").attr("value", AG.param.get("adding"));

            // disable gadget's  buttons to prevent overlapping calls
            $('.macro-button-add').attr("disabled", "disabled");
            $('.macro-button-remove').attr("disabled", "disabled");
            var gadgetUrl = button.siblings('.macro-hidden-uri').attr("value");
            addGadgetToDashboard(gadgetUrl, callbacks(button, addGadgetToDashboardSuccess, addGadgetToDashboardError));
            return false;
        };

        if (gadget.thumbnailUri && gadget.thumbnailUri != "") {
            gadgetDiv.prepend("<img src='" + gadget.thumbnailUri + "' alt='' width='120' height='60'/>");
        }

        if (gadget.description && gadget.description != "") {
            $(".macro-desc", gadgetDiv).append(gadget.description);
        } else {
            $(".macro-desc", gadgetDiv).append("<span class='unknown'>" + AG.param.get("descriptionNotAvailable") + "</span>");
        }

        if (gadget.authorName && gadget.authorName != "") {
            $(".macro-author", gadgetDiv).append(AJS.format(AG.param.get("gadgetAuthor"), "<a href='mailto:" + gadget.authorEmail + "'>" + gadget.authorName + "</a>"));
        } else {
            $(".macro-author", gadgetDiv).append("<span class='unknown'>" + AG.param.get("authorUnknown") + "</span>");
        }

        $(".macro-title", gadgetDiv).append("<a href=''>" + gadget.title + "</a>");
        $(".macro-title", gadgetDiv).click(onAdd);

        if (gadget.titleUri && gadget.titleUri != "") {
            $(".macro-title-uri-link", gadgetDiv).attr("href", gadget.titleUri);
        } else {
            $(".macro-title-uri", gadgetDiv).hide();
        }

        // add base url to the Gadget URL link if gadgetSpecUri is a relative path
        $(".macro-uri", gadgetDiv).attr("href", (gadget.gadgetSpecUri.match("^https?://") ? "" : AG.param.get("dashboardDirectoryBaseUrl"))
                                                + gadget.gadgetSpecUri);

        $(".macro-hidden-uri", gadgetDiv).attr("value", gadget.gadgetSpecUri);

        $(".macro-button-add", gadgetDiv).click(onAdd);

        // Directory Admins only
        if (AG.param.get("canAddExternalGadgetsToDirectory") == "true") {
            if (gadget.isDeletable) {
                // Make the button visible and hook it up to the remove function
                $(".macro-button-remove", gadgetDiv).css("display", "block").click(function(event) {
                    if (confirm(AG.param.get("removeGadget"))) {
                        var button = $(this);
                        button.attr("disabled", "disabled").attr("value", AG.param.get("removing"));
                        button.siblings('.macro-button-add').attr("disabled", "disabled");
                        var gadgetUrl = button.siblings('.macro-hidden-uri').attr("value");
                        removeGadgetFromDirectory(gadget.self, callbacks(button, removeGadgetFromDirectorySuccess, removeGadgetFromDirectoryError));
                    }
                    return false;
                });

            }
        }
    };

    // Construct the dialog without any data
    var constructBrowser = function() {
        var mb = AJS.MacroBrowser.dialog = new AJS.Dialog(860, 530, "macro-browser-dialog");
        mb.addHeader(AG.param.get("gadgetDirectory"));

        // Add buttons to page one of dialog if the user has perms
        if (AG.param.get("canAddExternalGadgetsToDirectory") == "true") {
            mb.addButton(AG.param.get("addByUrlButtonLabel"),
            function(dialog) {
                dialog.nextPage();
                $('#add-gadget-url').focus();
            },
            "add-by-url left");
        }

        if (!mb.page[0].buttonpanel) {
            mb.page[0].buttonpanel = AJS("div").addClass("button-panel");
            mb.page[0].element.append(mb.page[0].buttonpanel);
        }

        // Add Help Links
        var helpLink = AJS("span").attr("class", "directory-help-link");
        AJS("a").html(AG.param.get("helpLinkLearnMoreAboutGadgets")).css("margin-right", "10px").attr("href", AG.param.get("helpLinkLearnMoreAboutGadgetsUrl")).attr("target", "_blank").appendTo(helpLink);
        AJS("a").html(AG.param.get("helpLinkCreateYourOwnGadget")).css("margin-right", "10px").attr("href", AG.param.get("helpLinkCreateYourOwnGadgetUrl")).attr("target", "_blank").appendTo(helpLink);
        mb.page[0].buttonpanel.append(helpLink);

        mb.addButton(AG.param.get("finishButtonLabel"),
        function(dialog) {
            dialog.hide();
        },
        "finish");

        // Directory Admins only
        if (AG.param.get("canAddExternalGadgetsToDirectory") == "true") {
            // Add 2nd page and buttons - Add by URL
            mb.addPage().addPanel("addByUrl", $("#add-by-url-template")).addButton(AG.param.get("backButtonLabel"),
            function(dialog) {
                dialog.prevPage();
            },
            "back").addButton(AG.param.get("finishButtonLabel"),
            function(dialog) {
                dialog.hide();
            },
            "finish");
            mb.page[1].addHeader(AG.param.get("addGadgetByUrl"));
            // AG-917 : when the user hits 'enter' in the text box, click the add-gadget-submit button
            $('#add-gadget-url').keydown(function(e) {
               if (e.keyCode == 13) {
                   $('#add-gadget-submit').click();
               }
            });
            // Button for adding a gadget by URL
            $('#add-gadget-submit').click(function() {
                addGadgetToDirectory();
            });
        }

        // add search box to top right
        var searchInput =   AJS.$("<input type='search'/>").attr("id", "macro-browser-search").keyup(function(e) {
            var text = AJS.$(e.target).val();
            var macroSummaries = AJS.$("#macro-browser-dialog .panel-body .macro-list-item");
            var withText = ":containsIgnoreCase(" + text + ")";
            if (text !== "") {
                macroSummaries.filter(withText).show();
                macroSummaries.not(withText).hide();
            } else {
                resetSearchResults();
            }
        }).focus(function(e) {
            var searchInput = AJS.$(e.target);
            if (searchInput.hasClass("blank-search")) {
                searchInput.removeClass("blank-search").val("");
            }
        }).blur(function(e) {
            var searchInput = AJS.$(e.target);
            if (searchInput.val() == "") {
                searchInput.addClass("blank-search").val(AG.param.get("blankSearchText"));
            }
        }).blur();
        mb.page[0].header.prepend(searchInput);
    };

    // Loads the categories and macros into the dialog
    var loadBrowserContent = function(data, gadgetUrl) {
        var mb = AJS.MacroBrowser.dialog;

        // sort the categories and macros
        data.categories.sort(function(one, two) {
            return (one.name > two.name ? 1 : -1);
        });
        data.gadgets.sort(function(one, two) {
            return (one.title > two.title ? 1 : -1);
        });

        var makeCategoryList = function(id) {
            return $("#macro-summaries-template").clone().attr("id", "category-" + id);
        };

        // Create and fill each node that contains an item
        var makeGadgetSummary = function(gadget) {
            var macroDiv = AJS.MacroBrowser.clone("#macro-summary-template");
            fillMacroTemplate(macroDiv, gadget);
            return macroDiv;
        };

        // Initialize list of categories
        var categoryDivs = {
            all: makeCategoryList("all")
        };

        // Fill items on the right
        $(data.gadgets).each(function(i, gadget) {
            // Remove all characters that are not valid in an HTML Id.
            var title = gadget.title.replace(/[^A-Za-z0-9_]/g,'');
            var macroDiv = makeGadgetSummary(gadget).attr("id", "macro-" + title);
            categoryDivs.all.append(macroDiv);
            $([gadget.categories]).each(function(i, catKey) {
                if (catKey.constructor == Array) { // if there are multiple categories for this gadget, loop through all of them
                    for (var j = 0; j < catKey.length; ++j) {
                        categoryDivs[catKey[j]] = categoryDivs[catKey[j]] || makeCategoryList(j);
                        categoryDivs[catKey[j]].append(makeGadgetSummary(gadget).attr("id", catKey[j] + "-macro-" + title));
                    }
                } else {
                    categoryDivs[catKey] = categoryDivs[catKey] || makeCategoryList(j);
                    categoryDivs[catKey].append(makeGadgetSummary(gadget).attr("id", catKey + "-macro-" + title));
                }
            });
        });

        // Fill category menu on the left
        mb.page[0].addPanel(AG.param.get("all") + " (" + categoryDivs["all"].children().length + ")", categoryDivs["all"]);

        $(data.categories).each(function() {
            if (categoryDivs[this.name]) {
                mb.page[0].addPanel(this.name + " (" + categoryDivs[this.name].children().length + ")", categoryDivs[this.name], this.name);
            }
        });

        mb.page[0].gotoPanel(0, 0);
        $(mb.page).each( function() {
            this.recalcSize();
        });

        // Highlight and scroll to the added gadget
        if(gadgetUrl != null) {

            // Find the new Gadget
            var gadgetElement = $('.macro-hidden-uri[value='+gadgetUrl+']').parents('.macro-list-item');
            var gadgetId = gadgetElement.attr("id");

            scrollToGadget(gadgetId);

            // Clear the search field
            $('#add-gadget-url').attr('value', "");
        }


        mb.ready = true;
    };

    // Load data from Directory Resource
    var loadBrowser = function(displayDialog, gadgetUrl) {
        browserContentLoading = true;
        var xhr = $.ajax({
                type: "GET",
                dataType: "json",
                cache: false,
                url: dashboardDirectoryResourceUrl + ".json",
                success: function(data) {
                    browserContentLoading = false;
                    browserContentLoaded = true;
                    loadBrowserContent(data, gadgetUrl);
                    if(displayDialog) {
                        showDialog();
                    }
                    $('#add-gadget-submit').attr("value", AG.param.get("addGadget")).removeAttr("disabled");
                },
                error: function(request, textStatus, errorThrown) {
                    browserContentLoading = false;
                    if (request.status == 403 || request.status == 401) {
                        showError(AG.param.get("dashboardErrorDashboardPermissions"));
                    } else {
                        showError(AG.param.get("failedToLoadError"));
                    }
                    $('#add-gadget-submit').attr("value", AG.param.get("addGadget")).removeAttr("disabled");
                }
            });
        //only display the throbber if we're displaying the dialog.
        if(displayDialog) {
            $(xhr).throbber({target: $("#dash-throbber")});
        }
        $(xhr).throbber({target: $("#dir-throbber")});
    };

    // Clear Dialog Content
    var clearBrowser = function() {
        var mb = AJS.MacroBrowser.dialog;
        $(mb.page[0].panel).each(function() {
            this.body.remove();
        });
        $(mb.page[0].panel).each(function() {
            this.remove();
        });
        mb.page[0].curtab = 0;
    };

    var resetSearchResults = function () {
        AJS.$("#macro-browser-dialog .panel-body .macro-list-item").show();
    };

    var showDialog = function() {
        var mb = AJS.MacroBrowser;
        if (mb.dialog && mb.dialog.ready) {
            // bind escape to close dialog
            $(document).keyup(function(e) {
                if (e.keyCode == 27) {
                    mb.dialog.hide();
                    $(document).unbind("keyup", arguments.callee);
                    $("#macro-browser-search").val("");
                    return AJS.stopEvent(e);
                }
            });

            if (mb.selectedMacro) {
                AJS.log("selectedMacro: " + mb.selectedMacro.name);
                replicateSelectMacro(mb.selectedMacro.name);
            } else {
                mb.dialog.show(); // we must show then go to panel - this order is important for IE6
                mb.dialog.gotoPanel(0, 0);
                AJS.$("#macro-browser-search").addClass("blank-search").val(AG.param.get("blankSearchText"));
                resetSearchResults();
            }
        } else {
            showError(errorStatus);
        }
    };

    var showError = function(message) {
        alert(message); // TODO: AG-95 show this in the UI, not an alert.
    };

    // Initialize Everything
    constructBrowser();

    // Hook up the button
    $(".add-gadget-link").live("click", function(e) {

        AJS.activeColumn = 0;
        // button id = "add-gadget-#"
        if($(this).attr("id").substring(11)){
            AJS.activeColumn = $(this).attr("id").substring(11);
        }

        if (browserContentLoaded) {
            showDialog();
        } else if (!browserContentLoading) {
            loadBrowser(true);
        }
        return AJS.stopEvent(e);
    });
})(AJS.$);

//todo move this into AJS
AJS.$.extend(jQuery.expr[':'], {
    containsIgnoreCase: function(a, i, m) {
        return (a.textContent || a.innerText || jQuery(a).text() || '').toLowerCase().indexOf((m[3] || '').toLowerCase()) >= 0;
    }
});